@@ -28,6 +28,142 @@
 
 G_BEGIN_DECLS
 
+/**
+ * _GST_ELEMENT_REGISTER_DEFINE_BEGIN: (attributes doc.skip=true)
+ */
+#define _GST_ELEMENT_REGISTER_DEFINE_BEGIN(element) \
+G_BEGIN_DECLS \
+gboolean G_PASTE (gst_element_register_, element) (GstPlugin * plugin) \
+{ \
+  {
+
+/**
+ * _GST_ELEMENT_REGISTER_DEFINE_END: (attributes doc.skip=true)
+ */
+#define _GST_ELEMENT_REGISTER_DEFINE_END(element_name, rank, type) \
+  } \
+  return gst_element_register (plugin, element_name, rank, type); \
+} \
+G_END_DECLS
+
+/**
+ * GST_ELEMENT_REGISTER_DEFINE_CUSTOM:
+ * @element: The element name in lower case, with words separated by '_'.
+ * Used to generate `gst_element_register_*(GstPlugin* plugin)`.
+ * @register_func: pointer to a method with the format: `gboolean register_func (GstPlugin* plugin);`
+ *
+ * A convenience macro to define the entry point of an
+ * element `gst_element_register_*(GstPlugin* plugin)` which uses
+ * register_func as the main registration method for the element.
+ * As an example, you may define the element named "streamer-filter"
+ * with the namespace `my` as following using `element_register_custom`:
+ *
+ * ```
+ * GST_ELEMENT_REGISTER_DEFINE_CUSTOM (my_element, element_register_custom)
+ * ```
+ *
+ * Since: 1.20
+ */
+#define GST_ELEMENT_REGISTER_DEFINE_CUSTOM(element, register_func) \
+G_BEGIN_DECLS \
+gboolean G_PASTE (gst_element_register_, element) (GstPlugin * plugin) \
+{ \
+  return register_func (plugin); \
+} \
+G_END_DECLS
+
+/**
+ * GST_ELEMENT_REGISTER_DEFINE:
+ * @e: The element name in lower case, with words separated by '_'.
+ * Used to generate `gst_element_register_*(GstPlugin* plugin)`.
+ * @e_n: The public name of the element
+ * @r: The #GstRank of the element (higher rank means more importance when autoplugging, see #GstRank)
+ * @t: The #GType of the element.
+ *
+ * A convenience macro to define the entry point of an
+ * element `gst_element_register_*(GstPlugin* plugin)`.
+ * As an example, you may define the element named "streamer-filter"
+ * with the namespace `my` as following:
+ *
+ * ```
+ * GST_ELEMENT_REGISTER_REGISTER_DEFINE (stream_filter, "stream-filter", GST_RANK_PRIMARY, MY_TYPE_STREAM_FILTER)
+ * ```
+ *
+ * Since: 1.20
+ */
+#define GST_ELEMENT_REGISTER_DEFINE(e, e_n, r, t) _GST_ELEMENT_REGISTER_DEFINE_BEGIN(e) _GST_ELEMENT_REGISTER_DEFINE_END(e_n, r, t)
+
+/**
+ * GST_ELEMENT_REGISTER_DEFINE_WITH_CODE:
+ * @e: The element name in lower case, with words separated by '_'.
+ * Used to generate `gst_element_register_*(GstPlugin* plugin)`.
+ * @e_n: The public name of the element
+ * @r: The #GstRank of the element (higher rank means more importance when autoplugging, see #GstRank)
+ * @t: The #GType of the element.
+ * @_c_: Custom code that gets inserted in the gst_element_register_*() function.
+ *
+ * A convenience macro to define the entry point of an
+ * element `gst_element_register_*(GstPlugin* plugin)` executing code
+ * before gst_element_register in `gst_element_register_*(GstPlugin* plugin)`.
+
+ * As an example, you may define the element named "stream-filter"
+ * with the namespace `my` as following:
+ *
+ * ```
+ * #define _pre_register_init \
+ *   my_stream_filter_pre_register (plugin);
+ * GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (stream_filter, "stream-filter", GST_RANK_PRIMARY, MY_TYPE_STREAM_FILTER, _pre_register_init)
+ * ```
+ *
+ * Since: 1.20
+ */
+#define GST_ELEMENT_REGISTER_DEFINE_WITH_CODE(e, e_n, r, t, _c_) _GST_ELEMENT_REGISTER_DEFINE_BEGIN(e) {_c_;} _GST_ELEMENT_REGISTER_DEFINE_END(e_n, r, t)
+
+/**
+ * GST_ELEMENT_REGISTER_DECLARE:
+ * @element: The element name in lower case, with words separated by '_'.
+ *
+ * This macro can be used to declare a new element.
+ * It has to be used in combination with #GST_ELEMENT_REGISTER_DEFINE macros
+ * and must be placed outside any block to declare the element registration
+ * function.
+ * As an example, you may declare the element named "stream-filter"
+ * with the namespace `my` as following:
+ *
+ * ```
+ * GST_ELEMENT_REGISTER_DECLARE (stream_filter)
+ * ```
+ *
+ * Since: 1.20
+ */
+#define GST_ELEMENT_REGISTER_DECLARE(element) \
+G_BEGIN_DECLS \
+gboolean G_PASTE(gst_element_register_, element) (GstPlugin * plugin); \
+G_END_DECLS
+
+/**
+ * GST_ELEMENT_REGISTER:
+ * @element: The element name in lower case, with words separated by '_'.
+ * @plugin: The #GstPlugin where to register the element.
+ *
+ * This macro can be used to register an element into a #GstPlugin.
+ * This method will be usually called in the plugin init function
+ * but can also be called with a NULL plugin,
+ * for example with a static registration of the element.
+ * It has to be used in combination with #GST_ELEMENT_REGISTER_DECLARE.
+ *
+ * ```
+ * GstPlugin* plugin;
+ *
+ * ...
+ *
+ * GST_ELEMENT_REGISTER (stream_filter, plugin);
+ * ```
+ *
+ * Since: 1.20
+ */
+#define GST_ELEMENT_REGISTER(element, plugin) G_PASTE(gst_element_register_, element) (plugin)
+
 /* gstelement.h and gstelementfactory.h include each other */
 typedef struct _GstElement GstElement;
 typedef struct _GstElementClass GstElementClass;
@@ -348,7 +484,7 @@ typedef enum
 #define GST_ELEMENT_START_TIME(elem)            (GST_ELEMENT_CAST(elem)->start_time)
 
 GST_API
-GstStructure *gst_make_element_message_details (const char *name, ...);
+GstStructure *gst_make_element_message_details (const char *name, ...) G_GNUC_NULL_TERMINATED;
 
 #define GST_ELEMENT_MESSAGE_MAKE_DETAILS(args) gst_make_element_message_details args
 
@@ -370,6 +506,25 @@ G_STMT_START {                                                          \
       ("flow-return", G_TYPE_INT, flow_return, NULL));\
 } G_STMT_END
 
+/**
+ * GST_ELEMENT_FLOW_ERROR_WITH_LOG_CTX:
+ * @el:           the element that generates the error
+ * @log_ctx:      the log context
+ * @flow_return:  the GstFlowReturn leading to that ERROR message
+ *
+ * Utility function that elements can use in case they encountered a fatal
+ * data processing error due to wrong flow processing.
+ *
+ * Since: 1.28
+ */
+#define GST_ELEMENT_FLOW_ERROR_WITH_LOG_CTX(el,log_ctx,flow_return)  \
+G_STMT_START {                                                          \
+  GST_ELEMENT_ERROR_WITH_DETAILS_AND_LOG_CTX (el, STREAM, FAILED, log_ctx, \
+      ("Internal data stream error."), \
+      ("streaming stopped, reason %s (%d)", gst_flow_get_name (flow_return), flow_return), \
+      ("flow-return", G_TYPE_INT, flow_return, NULL));\
+} G_STMT_END
+
 /**
  * GST_ELEMENT_ERROR_WITH_DETAILS:
  * @el:     the element that generates the error
@@ -403,6 +558,40 @@ G_STMT_START {                                                          \
       GST_FUNCTION, __LINE__, GST_ELEMENT_MESSAGE_MAKE_DETAILS(args));  \
 } G_STMT_END
 
+/**
+ * GST_ELEMENT_ERROR_WITH_DETAILS_AND_LOG_CTX:
+ * @el:     the element that generates the error
+ * @domain: like CORE, LIBRARY, RESOURCE or STREAM (see [GstGError](gsterror))
+ * @code:   error code defined for that domain (see [GstGError](gsterror))
+ * @log_ctx: the log context
+ * @text:   the message to display (format string and args enclosed in
+            parentheses)
+ * @debug:  debugging information for the message (format string and args
+            enclosed in parentheses)
+ * @args:   optional name, type, value triplets, which will be stored
+ *          in the associated GstStructure. NULL terminator required.
+ *          Must be enclosed within parentheses.
+ *
+ * Utility function that elements can use in case they encountered a fatal
+ * data processing error. The pipeline will post an error message and the
+ * application will be requested to stop further media processing.
+ *
+ * Since: 1.28
+ */
+#define GST_ELEMENT_ERROR_WITH_DETAILS_AND_LOG_CTX(el,domain,code,log_ctx,text,debug,args)  \
+G_STMT_START {                                                          \
+  gchar *__txt = _gst_element_error_printf text;                        \
+  gchar *__dbg = _gst_element_error_printf debug;                       \
+  if (__txt)                                                            \
+    GST_CTX_WARNING_OBJECT (log_ctx, el, "error: %s", __txt);           \
+  if (__dbg)                                                            \
+    GST_CTX_WARNING_OBJECT (log_ctx, el, "error: %s", __dbg);           \
+  gst_element_message_full_with_details (GST_ELEMENT(el),               \
+    GST_MESSAGE_ERROR, GST_ ## domain ## _ERROR,                        \
+      GST_ ## domain ## _ERROR_ ## code, __txt, __dbg, __FILE__,        \
+      GST_FUNCTION, __LINE__, GST_ELEMENT_MESSAGE_MAKE_DETAILS(args));  \
+} G_STMT_END
+
 /**
  * GST_ELEMENT_ERROR:
  * @el:     the element that generates the error
@@ -431,6 +620,37 @@ G_STMT_START {                                                          \
       GST_FUNCTION, __LINE__);                                          \
 } G_STMT_END
 
+/**
+ * GST_ELEMENT_ERROR_WITH_LOG_CTX:
+ * @el:     the element that generates the error
+ * @domain: like CORE, LIBRARY, RESOURCE or STREAM (see [GstGError](gsterror))
+ * @code:   error code defined for that domain (see [GstGError](gsterror))
+ * @log_ctx: the log context
+ * @text:   the message to display (format string and args enclosed in
+            parentheses)
+ * @debug:  debugging information for the message (format string and args
+            enclosed in parentheses)
+ *
+ * Utility function that elements can use in case they encountered a fatal
+ * data processing error. The pipeline will post an error message and the
+ * application will be requested to stop further media processing.
+ *
+ * Since: 1.28
+ */
+#define GST_ELEMENT_ERROR_WITH_LOG_CTX(el,domain,code,log_ctx,text,debug)     \
+G_STMT_START {                                                          \
+  gchar *__txt = _gst_element_error_printf text;                        \
+  gchar *__dbg = _gst_element_error_printf debug;                       \
+  if (__txt)                                                            \
+    GST_CTX_WARNING_OBJECT (log_ctx, el, "error: %s", __txt);           \
+  if (__dbg)                                                            \
+    GST_CTX_WARNING_OBJECT (log_ctx, el, "error: %s", __dbg);           \
+  gst_element_message_full (GST_ELEMENT(el),                            \
+    GST_MESSAGE_ERROR, GST_ ## domain ## _ERROR,                        \
+      GST_ ## domain ## _ERROR_ ## code, __txt, __dbg, __FILE__,        \
+      GST_FUNCTION, __LINE__);                                          \
+} G_STMT_END
+
 /**
  * GST_ELEMENT_WARNING_WITH_DETAILS:
  * @el:     the element that generates the warning
@@ -464,6 +684,71 @@ G_STMT_START {                                                          \
     GST_FUNCTION, __LINE__, GST_ELEMENT_MESSAGE_MAKE_DETAILS(args));    \
 } G_STMT_END
 
+/**
+ * GST_ELEMENT_WARNING_WITH_DETAILS_AND_LOG_CTX:
+ * @el:     the element that generates the warning
+ * @domain: like CORE, LIBRARY, RESOURCE or STREAM (see [GstGError](gsterror))
+ * @code:   error code defined for that domain (see [GstGError](gsterror))
+ * @log_ctx: the log context
+ * @text:   the message to display (format string and args enclosed in
+            parentheses)
+ * @debug:  debugging information for the message (format string and args
+            enclosed in parentheses)
+ * @args:   optional name, type, value triplets, which will be stored
+ *          in the associated GstStructure. NULL terminator required.
+ *          Must be enclosed within parentheses.
+ *
+ * Utility function that elements can use in case they encountered a non-fatal
+ * data processing problem. The pipeline will post a warning message and the
+ * application will be informed.
+ *
+ * Since: 1.28
+ */
+#define GST_ELEMENT_WARNING_WITH_DETAILS_AND_LOG_CTX(el, domain, code, log_ctx, text, debug, args)\
+G_STMT_START {                                                          \
+  gchar *__txt = _gst_element_error_printf text;                        \
+  gchar *__dbg = _gst_element_error_printf debug;                       \
+  if (__txt)                                                            \
+    GST_CTX_WARNING_OBJECT (log_ctx, el, "warning: %s", __txt);         \
+  if (__dbg)                                                            \
+    GST_CTX_WARNING_OBJECT (log_ctx, el, "warning: %s", __dbg);         \
+  gst_element_message_full_with_details (GST_ELEMENT(el),               \
+    GST_MESSAGE_WARNING, GST_ ## domain ## _ERROR,                      \
+    GST_ ## domain ## _ERROR_ ## code, __txt, __dbg, __FILE__,          \
+    GST_FUNCTION, __LINE__, GST_ELEMENT_MESSAGE_MAKE_DETAILS(args));    \
+} G_STMT_END
+
+/**
+ * GST_ELEMENT_WARNING_WITH_LOG_CTX:
+ * @el:     the element that generates the warning
+ * @domain: like CORE, LIBRARY, RESOURCE or STREAM (see [GstGError](gsterror))
+ * @code:   error code defined for that domain (see [GstGError](gsterror))
+ * @log_ctx: the log context
+ * @text:   the message to display (format string and args enclosed in
+            parentheses)
+ * @debug:  debugging information for the message (format string and args
+            enclosed in parentheses)
+ *
+ * Utility function that elements can use in case they encountered a non-fatal
+ * data processing problem. The pipeline will post a warning message and the
+ * application will be informed.
+ *
+ * Since: 1.28
+ */
+#define GST_ELEMENT_WARNING_WITH_LOG_CTX(el, domain, code, log_ctx, text, debug)              \
+G_STMT_START {                                                          \
+  gchar *__txt = _gst_element_error_printf text;                        \
+  gchar *__dbg = _gst_element_error_printf debug;                       \
+  if (__txt)                                                            \
+    GST_CTX_WARNING_OBJECT (log_ctx, el, "warning: %s", __txt);         \
+  if (__dbg)                                                            \
+    GST_CTX_WARNING_OBJECT (log_ctx, el, "warning: %s", __dbg);         \
+  gst_element_message_full (GST_ELEMENT(el),                            \
+    GST_MESSAGE_WARNING, GST_ ## domain ## _ERROR,                      \
+    GST_ ## domain ## _ERROR_ ## code, __txt, __dbg, __FILE__,          \
+    GST_FUNCTION, __LINE__);                                            \
+} G_STMT_END
+
 /**
  * GST_ELEMENT_WARNING:
  * @el:     the element that generates the warning
@@ -528,6 +813,43 @@ G_STMT_START {                                                          \
     GST_FUNCTION, __LINE__, GST_ELEMENT_MESSAGE_MAKE_DETAILS(args));    \
 } G_STMT_END
 
+/**
+ * GST_ELEMENT_INFO_WITH_DETAILS_AND_LOG_CTX:
+ * @el:     the element that generates the information
+ * @domain: like CORE, LIBRARY, RESOURCE or STREAM (see [GstGError](gsterror))
+ * @code:   error code defined for that domain (see [GstGError](gsterror))
+ * @log_ctx: the log context
+ * @text:   the message to display (format string and args enclosed in
+            parentheses)
+ * @debug:  debugging information for the message (format string and args
+            enclosed in parentheses)
+ * @args:   optional name, type, value triplets, which will be stored
+ *          in the associated GstStructure. NULL terminator required.
+ *          Must be enclosed within parentheses.
+ *
+ * Utility function that elements can use in case they want to inform
+ * the application of something noteworthy that is not an error.
+ * The pipeline will post a info message and the
+ * application will be informed.
+ * Optional name, type, value triplets may be supplied, and will be stored
+ * in the associated GstStructure. NULL terminator required.
+ *
+ * Since: 1.28
+ */
+#define GST_ELEMENT_INFO_WITH_DETAILS_AND_LOG_CTX(el, domain, code, log_ctx, text, debug, args)   \
+G_STMT_START {                                                          \
+  gchar *__txt = _gst_element_error_printf text;                        \
+  gchar *__dbg = _gst_element_error_printf debug;                       \
+  if (__txt)                                                            \
+    GST_CTX_INFO_OBJECT (log_ctx, el, "info: %s", __txt);               \
+  if (__dbg)                                                            \
+    GST_CTX_INFO_OBJECT (log_ctx, el, "info: %s", __dbg);               \
+  gst_element_message_full_with_details (GST_ELEMENT(el),               \
+    GST_MESSAGE_INFO, GST_ ## domain ## _ERROR,                         \
+    GST_ ## domain ## _ERROR_ ## code, __txt, __dbg, __FILE__,          \
+    GST_FUNCTION, __LINE__, GST_ELEMENT_MESSAGE_MAKE_DETAILS(args));    \
+} G_STMT_END
+
 /**
  * GST_ELEMENT_INFO:
  * @el:     the element that generates the information
@@ -557,6 +879,38 @@ G_STMT_START {                                                          \
     GST_FUNCTION, __LINE__);                                            \
 } G_STMT_END
 
+/**
+ * GST_ELEMENT_INFO_WITH_LOG_CTX:
+ * @el:     the element that generates the information
+ * @domain: like CORE, LIBRARY, RESOURCE or STREAM (see [GstGError](gsterror))
+ * @code:   error code defined for that domain (see [GstGError](gsterror))
+ * @log_ctx: the log context
+ * @text:   the message to display (format string and args enclosed in
+            parentheses)
+ * @debug:  debugging information for the message (format string and args
+            enclosed in parentheses)
+ *
+ * Utility function that elements can use in case they want to inform
+ * the application of something noteworthy that is not an error.
+ * The pipeline will post a info message and the
+ * application will be informed.
+ *
+ * Since: 1.28
+ */
+#define GST_ELEMENT_INFO_WITH_LOG_CTX(el, domain, code, log_ctx, text, debug)  \
+G_STMT_START {                                                          \
+  gchar *__txt = _gst_element_error_printf text;                        \
+  gchar *__dbg = _gst_element_error_printf debug;                       \
+  if (__txt)                                                            \
+    GST_CTX_INFO_OBJECT (log_ctx, el, "info: %s", __txt);               \
+  if (__dbg)                                                            \
+    GST_CTX_INFO_OBJECT (log_ctx, el, "info: %s", __dbg);               \
+  gst_element_message_full (GST_ELEMENT(el),                            \
+    GST_MESSAGE_INFO, GST_ ## domain ## _ERROR,                         \
+    GST_ ## domain ## _ERROR_ ## code, __txt, __dbg, __FILE__,          \
+    GST_FUNCTION, __LINE__);                                            \
+} G_STMT_END
+
 /* the state change mutexes and conds */
 /**
  * GST_STATE_GET_LOCK:
@@ -840,10 +1194,10 @@ GType                   gst_element_get_type            (void);
 /* clocking */
 
 GST_API
-GstClock*               gst_element_provide_clock       (GstElement *element);
+GstClock*               gst_element_provide_clock       (GstElement *element) G_GNUC_WARN_UNUSED_RESULT;
 
 GST_API
-GstClock*               gst_element_get_clock           (GstElement *element);
+GstClock*               gst_element_get_clock           (GstElement *element) G_GNUC_WARN_UNUSED_RESULT;
 
 GST_API
 gboolean                gst_element_set_clock           (GstElement *element, GstClock *clock);
@@ -872,7 +1226,7 @@ GST_API
 void                    gst_element_set_bus             (GstElement * element, GstBus * bus);
 
 GST_API
-GstBus *                gst_element_get_bus             (GstElement * element);
+GstBus *                gst_element_get_bus             (GstElement * element) G_GNUC_WARN_UNUSED_RESULT;
 
 /* context */
 
@@ -880,13 +1234,13 @@ GST_API
 void                    gst_element_set_context         (GstElement * element, GstContext * context);
 
 GST_API
-GList *                 gst_element_get_contexts        (GstElement * element);
+GList *                 gst_element_get_contexts        (GstElement * element) G_GNUC_WARN_UNUSED_RESULT;
 
 GST_API
-GstContext *            gst_element_get_context         (GstElement * element, const gchar * context_type);
+GstContext *            gst_element_get_context         (GstElement * element, const gchar * context_type) G_GNUC_WARN_UNUSED_RESULT;
 
 GST_API
-GstContext *            gst_element_get_context_unlocked (GstElement * element, const gchar * context_type);
+GstContext *            gst_element_get_context_unlocked (GstElement * element, const gchar * context_type) G_GNUC_WARN_UNUSED_RESULT;
 
 /* pad management */
 
@@ -900,25 +1254,28 @@ GST_API
 void                    gst_element_no_more_pads        (GstElement *element);
 
 GST_API
-GstPad*                 gst_element_get_static_pad      (GstElement *element, const gchar *name);
+GstPad*                 gst_element_get_static_pad      (GstElement *element, const gchar *name) G_GNUC_WARN_UNUSED_RESULT;
+
+GST_API G_DEPRECATED_FOR(gst_element_request_pad_simple)
+GstPad*                 gst_element_get_request_pad     (GstElement *element, const gchar *name) G_GNUC_WARN_UNUSED_RESULT;
 
 GST_API
-GstPad*                 gst_element_get_request_pad     (GstElement *element, const gchar *name);
+GstPad*                 gst_element_request_pad_simple  (GstElement *element, const gchar *name) G_GNUC_WARN_UNUSED_RESULT;
 
 GST_API
 GstPad*                 gst_element_request_pad         (GstElement *element, GstPadTemplate *templ,
-							 const gchar * name, const GstCaps *caps);
+							 const gchar * name, const GstCaps *caps) G_GNUC_WARN_UNUSED_RESULT;
 GST_API
 void                    gst_element_release_request_pad (GstElement *element, GstPad *pad);
 
 GST_API
-GstIterator *           gst_element_iterate_pads        (GstElement * element);
+GstIterator *           gst_element_iterate_pads        (GstElement * element) G_GNUC_WARN_UNUSED_RESULT;
 
 GST_API
-GstIterator *           gst_element_iterate_src_pads    (GstElement * element);
+GstIterator *           gst_element_iterate_src_pads    (GstElement * element) G_GNUC_WARN_UNUSED_RESULT;
 
 GST_API
-GstIterator *           gst_element_iterate_sink_pads   (GstElement * element);
+GstIterator *           gst_element_iterate_sink_pads   (GstElement * element) G_GNUC_WARN_UNUSED_RESULT;
 
 /**
  * GstElementForeachPadFunc:
@@ -1028,10 +1385,13 @@ void                    gst_element_lost_state          (GstElement * element);
  * @user_data: Data passed in the function where that callback has been passed
  *
  * Callback prototype used in #gst_element_call_async
+ *
+ * Deprecated: 1.28: Use #GstObjectCallAsyncFunc with gst_object_call_async()
+ * or #GstCallAsyncFunc with gst_call_async() instead.
  */
 typedef void          (*GstElementCallAsyncFunc)        (GstElement * element,
                                                          gpointer     user_data);
-GST_API
+GST_DEPRECATED_FOR(gst_object_call_async)
 void                    gst_element_call_async          (GstElement * element,
                                                          GstElementCallAsyncFunc func, gpointer user_data,
                                                          GDestroyNotify destroy_notify);
